
# Includes
source config.tcl
source http.tcl
source github.tcl
source irc.tcl

package require irc

# Irc stuff
::irc::config logger 1
::irc::config debug 1

set running 1
set irc ""
set pongtime [clock milliseconds]

proc render-issue {issue} {
		set issue_num [issue-number $issue]
		set summary [issue-summary $issue]
		set url [issue-url $issue]
		return "[color royal]([color blue]$url[color royal]): [color]$summary"
}

proc connect {} {
	global irc
	if {[catch {
			set irc [::irc::connection]
			$irc registerevent PRIVMSG {privmsg-handler [msg]}
			$irc registerevent PONG pong-handler
			$irc connect $::server
			$irc user $::nick 0 0 0
			$irc nick $::nick
			after 1000
			$irc join $::channel
			puts Connected.
			set ::pongtime [clock milliseconds]
		}]} {
			after 60000 connect
		}
}

proc ping {} {
	$::irc serverping
	puts "Pinging server"
	after 120000 ping
}

proc check-pong {} {
	if {($::pongtime + 240000) < [clock milliseconds]} {
		set ::pongtime [clock milliseconds]
		$::irc destroy
		after 10000
		connect
	}
	after 1000 check-pong
}
check-pong

proc privmsg-handler {msg} {
	foreach word $msg {
		if {[string index $word 0] == "#"} {
			set issue_num [string range $word 1 end]
			if {![string is integer $issue_num]} return
			$::irc privmsg $::channel [render-issue $issue]
		}
	}

	if {$msg == "!quit"} {
		set ::running 0
		puts Quitting.
	}

	if {[lindex $msg 0] == "${::nick}:"} {
		$::irc privmsg $::channel "I'm a bot :P"
	}
}

proc pong-handler {} {
	set ::pongtime [clock milliseconds]
}

proc new-issues {} {
	set interval 60
	set tz_diff [expr {60 * 60 * 4}]
	set issues [get-last-issues [expr {$interval + $tz_diff}]]
	foreach issue $issues {
		if {[issue-new? $issue]} {
			set tag "New:"
		} elseif {[issue-closed? $issue]} {
			set tag "Closed:"
		} else {
			set tag "Updated:"
		}
		if {$::irc != ""} {
			$::irc privmsg $::channel "$tag [render-issue $issue]"
		}
	}

	after 60000 new-issues
}
new-issues

connect
ping
vwait running
$irc destroy
exit

